using System.Collections.Generic;
using UnityEngine;
using SimpleJSON;
using System.Collections.Generic;
using UnityEngine.Events;

/* 95% written by MeshedVR
Doc'0c adapted it to a plugin
*/
namespace ReVAMped
{
    public class WindControl : MVRScript
    {
        Atom windDirection;
        public static Vector3 globalWind = Vector3.zero;
        protected string _missingReceiverStoreId = string.Empty;
        public bool isGlobal;
		public bool isAvatar;
        protected JSONStorableBool isGlobalJSON;
		protected JSONStorableBool isAvatarJSON;
        protected Atom receivingAtom;
        protected JSONStorableStringChooser atomJSON;
        protected JSONStorable receiver;
        protected JSONStorableStringChooser receiverJSON;
        protected string _receiverTargetName;
        protected JSONStorableVector3 receiverTarget;
        protected JSONStorableStringChooser receiverTargetJSON;
        protected JSONStorableFloat currentMagnitudeJSON;
        protected JSONStorableBool autoJSON;
        protected JSONStorableFloat periodJSON;
        protected JSONStorableFloat quicknessJSON;
        protected JSONStorableFloat windDecay;
        protected JSONStorableFloat avatarMobility;
        protected JSONStorableFloat windStrength;
        protected JSONStorableFloat targetMagnitudeJSON;
        protected JSONStorableFloat lowerMagnitudeJSON;
        protected JSONStorableFloat upperMagnitudeJSON;
        protected float timer;
        protected Dictionary<string, ForceReceiver> receiverNameToForceReceiver;
        protected Rigidbody RB;

        protected void SyncReceiverRB(string receiver)
        {
            if (receiver != null)
            {
                ForceReceiver fr;
                if (receiverNameToForceReceiver.TryGetValue(receiver, out fr))
                {
                    RB = fr.GetComponent<Rigidbody>();
                }
                else
                {
                    RB = null;
                }
            }
            else
            {
                RB = null;
            }
        }
        
        protected void SyncIsGlobal(bool b)
        {
            isGlobal = b;
        }
		
		protected void SyncIsAvatar(bool b)
        {

            isAvatar = b;
            /*
            if (isAvatar)
			{

                setControllerStateComply("pelvisControl");
                setControllerStateComply("rNippleControl");
                setControllerStateComply("lNippleControl");
                //setControllerStateComply("rKneeControl");
                //setControllerStateComply("lKneeControl");
                setControllerStateComply("rArmControl");
                setControllerStateComply("lArmControl");
                setControllerStateComply("rElbowControl");
                setControllerStateComply("lElbowControl");
            }
            if (!isAvatar)
            {

                setControllerStateOff("pelvisControl");
                setControllerStateOff("rNippleControl");
                setControllerStateOff("lNippleControl");
                //setControllerStateOff("rKneeControl");
                //setControllerStateOff("lKneeControl");
                setControllerStateOff("rArmControl");
                setControllerStateOff("lArmControl");
                setControllerStateOff("rElbowControl");
                setControllerStateOff("lElbowControl");
            }
            */
        }

        protected void SyncAtomChoices()
        {
            List<string> stringList = new List<string>();
            stringList.Add("None");
            foreach (string atomUiD in SuperController.singleton.GetAtomUIDs())
                stringList.Add(atomUiD);
            atomJSON.choices = stringList;
        }

        protected void OnAtomRename(string oldid, string newid)
        {
            SyncAtomChoices();
            if (atomJSON == null || !(receivingAtom != null))
                return;
            atomJSON.valNoCallback = receivingAtom.uid;
        }

        protected void SyncAtom(string atomUID)
        {
            List<string> stringList = new List<string>();
            stringList.Add("None");
            if (atomUID != null)
            {
                receivingAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (receivingAtom != null)
                {
                    foreach (string storableId in receivingAtom.GetStorableIDs())
                    {
                        stringList.Add(storableId);
                    }
                    receiverNameToForceReceiver = new Dictionary<string, ForceReceiver>();
                    foreach (ForceReceiver fr in receivingAtom.forceReceivers)
                    {
                        receiverNameToForceReceiver.Add(fr.name, fr);
                    }
                }
            }
            else
                receivingAtom = (Atom)null;

            receiverJSON.choices = stringList;
            receiverJSON.val = "None";
        }

        protected void CheckMissingReceiver()
        {
            if (!(_missingReceiverStoreId != string.Empty)
                || !(receivingAtom != null)
                || !(receivingAtom.GetStorableByID(_missingReceiverStoreId) != null))
                return;

            string receiverTargetName = _receiverTargetName;
            SyncReceiver(_missingReceiverStoreId);
            _missingReceiverStoreId = string.Empty;
            insideRestore = true;
            receiverTargetJSON.val = receiverTargetName;
            insideRestore = false;
        }

        protected void SyncReceiver(string receiverID)
        {
            List<string> stringList = new List<string>();
            stringList.Add("None");
            if (receivingAtom != null && receiverID != null)
            {
                receiver = receivingAtom.GetStorableByID(receiverID);
                if (receiver != null)
                {
                    foreach (string vector3ParamName in receiver.GetVector3ParamNames())
                    {
                        stringList.Add(vector3ParamName);
                    }
                }
                else if (receiverID != "None")
                    _missingReceiverStoreId = receiverID;
            }
            else
            {
                receiver = null;
            }

            receiverTargetJSON.choices = stringList;
            receiverTargetJSON.val = "None";
        }

        protected void SyncReceiverTarget(string receiverTargetName)
        {
            _receiverTargetName = receiverTargetName;
            receiverTarget = (JSONStorableVector3)null;
            if (receiver != null && receiverTargetName != null)
                receiverTarget = receiver.GetVector3JSONParam(receiverTargetName);
        }

        public override void Init()
        {
            try
            {

                if ((bool)(SuperController.singleton))
                    SuperController.singleton.onAtomUIDRenameHandlers += new SuperController.OnAtomUIDRename(OnAtomRename);

                isGlobalJSON = new JSONStorableBool("Affect Cloth/Hair", false, new JSONStorableBool.SetBoolCallback(SyncIsGlobal));
				isAvatarJSON = new JSONStorableBool("Affect Avatar", false, new JSONStorableBool.SetBoolCallback(SyncIsAvatar));

                atomJSON = new JSONStorableStringChooser(
                  "atom", SuperController.singleton.GetAtomUIDs(), (string)null,
                  "Atom", new JSONStorableStringChooser.SetStringCallback(SyncAtom));

                atomJSON.popupOpenCallback = new JSONStorableStringChooser.PopupOpenCallback(SyncAtomChoices);
                receiverJSON = new JSONStorableStringChooser(
                  "receiver", (List<string>)null, (string)null,
                  "Receiver", new JSONStorableStringChooser.SetStringCallback(SyncReceiver));


                receiverTargetJSON = new JSONStorableStringChooser(
                  "receiverTarget", (List<string>)null, (string)null,
                  "Target", new JSONStorableStringChooser.SetStringCallback(SyncReceiverTarget));

                currentMagnitudeJSON = new JSONStorableFloat("current Magnitude", 0.0f, -50f, 50f, false, true);
                autoJSON = new JSONStorableBool("auto", true);
                periodJSON = new JSONStorableFloat("period", 0.03f, 0.0f, 10f, false, true);
                quicknessJSON = new JSONStorableFloat("quickness", 30f, 0.0f, 100f, true, true);
                lowerMagnitudeJSON = new JSONStorableFloat("lowerMagnitude", -50f, -50f, 50f, false, true);
                upperMagnitudeJSON = new JSONStorableFloat("upperMagnitude", 150f, -50f, 150f, false, true);
                targetMagnitudeJSON = new JSONStorableFloat("targetMagnitude", 0.0f, -50f, 50f, false, false);
                windDecay = new JSONStorableFloat("Wind Decay", 1f, 1f, 100f, false, true);
                avatarMobility = new JSONStorableFloat("Avatar Mobility", 2f, 1f, 10, true, true);
                windStrength = new JSONStorableFloat("Wind Strength", 0f, 0.0f, 300f, false, true);

				RegisterBool(isAvatarJSON);
                RegisterBool(isGlobalJSON);
                RegisterStringChooser(atomJSON);
                //RegisterStringChooser(receiverJSON);
                //RegisterStringChooser(receiverTargetJSON);

                RegisterBool(autoJSON);
                RegisterFloat(periodJSON);
                RegisterFloat(quicknessJSON);
                RegisterFloat(upperMagnitudeJSON);
                RegisterFloat(lowerMagnitudeJSON);
                RegisterFloat(currentMagnitudeJSON);
                RegisterFloat(windStrength);
                RegisterFloat(avatarMobility);

                CreateToggle(isAvatarJSON);
                CreateToggle(isGlobalJSON);
                CreateScrollablePopup(atomJSON, true);
                //CreateScrollablePopup(receiverJSON, true).popupPanelHeight = 1100f;
                //CreateScrollablePopup(receiverTargetJSON, true);

                CreateSlider(currentMagnitudeJSON);
                CreateToggle(autoJSON);
                //CreateSlider(periodJSON);
                //CreateSlider(quicknessJSON);
                //CreateSlider(lowerMagnitudeJSON);
                //CreateSlider(upperMagnitudeJSON);
                //CreateSlider(targetMagnitudeJSON);
                CreateSlider(windStrength);
                //CreateSlider(windDecay);
                CreateSlider(avatarMobility);

            }
            catch (System.Exception e)
            {
                SuperController.LogError("" + e);
            }
        }


        protected void Update()
        {
            if (currentMagnitudeJSON == null)
                return;
            if (autoJSON != null && autoJSON.val)
            {
                timer -= Time.deltaTime;
                if ((double)timer < 0.0)
                {
                    timer = periodJSON.val;
                    lowerMagnitudeJSON.val=-(windStrength.val/4);
                    upperMagnitudeJSON.val=windStrength.val;
                    quicknessJSON.val=windStrength.val/5;
                    periodJSON.val=0.2f-(windStrength.val/1000);
                    if(periodJSON.val<0.03f)
                    {
                        periodJSON.val=0.03f;
                    }
                    targetMagnitudeJSON.val = Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val);
                }
                if(windStrength.val==0)
                {
                    currentMagnitudeJSON.val=0;
                }
                else
                {
                currentMagnitudeJSON.val = Mathf.Lerp(currentMagnitudeJSON.val, targetMagnitudeJSON.val, Time.deltaTime * quicknessJSON.val);
                }
            }
            Vector3 vector3 = currentMagnitudeJSON.val * transform.forward;
            if (isGlobal)
            {
                
				//if (receivingAtom == null)
                    //return;
                
                
                foreach (string storableId in receivingAtom.GetStorableIDs())
				{
					SyncReceiver(storableId);
                    
                    
					foreach (string vector3ParamName in receiver.GetVector3ParamNames())
					{
						SyncReceiverTarget(vector3ParamName);
						receiverTarget.val = currentMagnitudeJSON.val * transform.forward;
					}
                    
                }
                
            }
            
			if (isAvatar)
			{
                /*
                updateController("pelvisControl", 5);
                updateController("rNippleControl", 2);
                updateController("lNippleControl", 2);
                //updateController("rKneeControl", 1);
                //updateController("lKneeControl", 1);
                updateController("rArmControl", 1);
                updateController("lArmControl", 1);
                updateController("rElbowControl", 1);
                updateController("lElbowControl", 1);
                //updateController("lHandControl", 1);
                //updateController("lFootControl", 1);
                //updateController("rFootControl", 1);
                */
                updateController("rNipple", 0.5f);
                updateController("lNipple", 0.5f);
                updateController("pelvis", 0.02f);
                updateController("rArm", 0.1f);
                updateController("lArm", 0.1f);
                updateController("rElbow", 0.1f);
                updateController("lElbow", 0.1f);
			}

        }

        protected void updateController(string Controller, float mobility)
        {
            SyncReceiverRB(Controller);
            //RB.transform.position+=((((currentMagnitudeJSON.val+Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val))/(10000/mobility*avatarMobility.val))) * (transform.forward));
            RB.AddForce((currentMagnitudeJSON.val+Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val))/(10*mobility*avatarMobility.val) * (transform.forward));
        }

        protected void setControllerStateOff(string controller)
        {
            FreeControllerV3 headControl = receivingAtom.GetStorableByID(controller) as FreeControllerV3;
            headControl.currentPositionState=FreeControllerV3.PositionState.Off;
            headControl.currentRotationState=FreeControllerV3.RotationState.Off;
        }

        protected void setControllerStateComply(string controller)
        {
            FreeControllerV3 headControl = receivingAtom.GetStorableByID(controller) as FreeControllerV3;
            headControl.currentPositionState=FreeControllerV3.PositionState.Comply;
            headControl.currentRotationState=FreeControllerV3.RotationState.Comply;
        }

        protected void OnDestroy()
        {
            if (SuperController.singleton)
                SuperController.singleton.onAtomUIDRenameHandlers -= new SuperController.OnAtomUIDRename(OnAtomRename);
        }
    }

}
